// Projection d'une image sur une sphere

#include "vect.h"
#include "imcarree.h"
#include <Imagine/Graphics.h>
#include <iostream>
using namespace Imagine;
using namespace std;

// Projette l'image ini (du plan z=0) a partir du point C sur la sphere
void projette(imcarree im, imcarree ini, vect C) {
    vect P = {im.cote/2, 0, 0};
    float rayon = norme(P-C);
    int w = im.cote;
    for(P.y=0; P.y<w; P.y++)
        for(P.x=0; P.x<w; P.x++) {
            im.r[(int)P.y*w+(int)P.x] = 255;
            im.g[(int)P.y*w+(int)P.x] = 255;
            im.b[(int)P.y*w+(int)P.x] = 255;
            vect V = P-C;
            P.z = rayon*rayon - (V.x*V.x+V.y*V.y);
            if(P.z >=0) {
                P.z = sqrt(P.z); // Projection sur sphere
                float lambda = -C.z / (P.z-C.z);
                vect Q = C+lambda*(P-C);
                int i = int(Q.x+0.5f);
                int j = int(Q.y+0.5f);
                if(0<=i && i<ini.cote && 0<=j && j<ini.cote) {
                    im.r[(int)P.y*w+(int)P.x] = ini.r[j*ini.cote+i];
                    im.g[(int)P.y*w+(int)P.x] = ini.g[j*ini.cote+i];
                    im.b[(int)P.y*w+(int)P.x] = ini.b[j*ini.cote+i];
                }
            }
        }
    putColorImage(0,0, im.r,im.g,im.b, w,w);
}

int main() {
    byte *r, *g, *b;
    int w, h;
    const char* fichier = srcPath("forty-licks.png");
    if(! loadColorImage(fichier, r,g,b, w,h)) {
        cerr << "Error loading " << fichier << endl;
        return 1;
    }

    imcarree im = decoupe(r,g,b, w,h);
    w = im.cote;

    openWindow(w,w);
    putColorImage(0,0, r,g,b, w,w);

    imcarree sphere;
    sphere.r = new byte[w*w];
    sphere.g = new byte[w*w];
    sphere.b = new byte[w*w];
    sphere.cote = w;

    cout << "Clic gauche pour placer le center" << endl;
    cout << "Clic droite pour regler la profondeur" << endl;

    vect C = {w/2, w/2, -w/2}; // Centre initial
    while(true) {
        int x, y;
        int mode = getMouse(x,y);
        if(mode == 1) {
            C.x = x; C.y = y;
            projette(sphere, im, C);
        } else if(mode ==3) {
            C.z = -y;
            projette(sphere, im, C);
        } else
            break;
    }
    endGraphics();
    delete [] im.r; delete [] im.g; delete [] im.b;
    delete [] sphere.r; delete [] sphere.g; delete [] sphere.b;
    return 0;
}
